package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ClientPrintStatisticsDto;
import com.qmth.distributed.print.business.bean.dto.ClientPrintStatisticsTotalDto;
import com.qmth.distributed.print.business.bean.dto.ClientPrintTaskDto;
import com.qmth.distributed.print.business.bean.dto.ClientPrintTaskTotalDto;
import com.qmth.distributed.print.business.bean.params.DeleteParams;
import com.qmth.distributed.print.business.bean.params.PrintPlanParams;
import com.qmth.distributed.print.business.bean.result.PrintPlanBrief;
import com.qmth.distributed.print.business.bean.result.PrintPlanResult;
import com.qmth.distributed.print.business.bean.result.TemplatePrintInfoResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ClassifyEnum;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.TemplateTypeEnum;
import com.qmth.distributed.print.business.mapper.ExamPrintPlanMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 印刷计划管理 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamPrintPlanServiceImpl extends ServiceImpl<ExamPrintPlanMapper, ExamPrintPlan> implements ExamPrintPlanService {
    @Resource
    private ExamPrintPlanMapper examPrintPlanMapper;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private BasicTemplateOrgService basicTemplateOrgService;
    @Resource
    private BasicSchoolService basicSchoolService;

    @Autowired
    private CommonService commonService;

    @Autowired
    private BasicAttachmentService basicAttachmentService;

    @Resource
    private BasicExamRuleService basicExamRuleService;

    @Resource
    private TBTaskService tbTaskService;

    @Autowired
    private ClientPrintDataService clientPrintDataService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<PrintPlanResult> printPlanPage(Long schoolId, Long printPlanId, PrintPlanStatusEnum status, Long startTime, Long endTime, int pageNumber, int pageSize) {
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        IPage<PrintPlanResult> page = examPrintPlanMapper.findPrintPlanPage(new Page<>(pageNumber, pageSize), schoolId, printPlanId, status, startTime, endTime, orgIds);
        List<PrintPlanResult> list = page.getRecords();
        for (PrintPlanResult printPlanResult : list) {
            List<Map> variableContent = JSONObject.parseArray(printPlanResult.getVariableContentTemp(), Map.class);
            List<Map> ordinaryContent = JSONObject.parseArray(printPlanResult.getOrdinaryContentTemp(), Map.class);
            printPlanResult.setVariableContent(variableContent);
            printPlanResult.setOrdinaryContent(ordinaryContent);

            double totalSubjects = examDetailService.calculateTotalSubjects(printPlanResult.getId());
            double totalPackages = examDetailService.calculateTotalPackages(printPlanResult.getId());
            printPlanResult.setTotalSubjects((int) totalSubjects);
            printPlanResult.setTotalPackages((int) totalPackages);
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<PrintPlanBrief> printPlanBriefQuery() {
        /*List<PrintPlanBrief> printPlanBriefList = new ArrayList<>();
        List<ExamPrintPlan> examPrintPlanList = this.list();
        for (ExamPrintPlan examPrintPlan : examPrintPlanList) {
            Long id = examPrintPlan.getId();
            String name = examPrintPlan.getName();
            PrintPlanStatusEnum status = examPrintPlan.getStatus();
            printPlanBriefList.add(new PrintPlanBrief(id, name ,status));
        }*/
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        List<PrintPlanBrief> list = this.baseMapper.list(schoolId, orgIds);
        return list;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> printPlanTemplateDatasource(Long requestSchoolId, Long userId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        //1.根据userId，schoolId查出orgId(应该从requestUser 获取)
        Long schoolId = sysUser.getSchoolId();
        Long orgId = sysUser.getOrgId();

        //2.根据orgId查询他的所有叶子id 并把这些id放入List<Long> ids
        List<Long> orgIds;
        if ((orgId == null || orgId == 0) && (schoolId == null || schoolId == 0)) {
            // 超级管理员权限
            orgIds = sysOrgService.list().stream().map(SysOrg::getId).collect(Collectors.toList());
        } else if ((orgId == null || orgId == 0) && schoolId > 0) {
            // 学校管理员权限
            orgIds = sysOrgService.list(new QueryWrapper<SysOrg>().lambda().eq(SysOrg::getSchoolId, schoolId)).stream().map(SysOrg::getId).collect(Collectors.toList());
        } else {
            // 普通权限
            List<Long> ids = new ArrayList<>();
            ids.add(orgId);
            orgIds = this.dp(ids, schoolId, new ArrayList<>());
            System.out.println("ordIds = " + orgIds);
        }

        //3.关联template和template_org表 查询出来orgIds所包含的所有模板并组装
        List<TemplatePrintInfoResult> templateInfoList = basicTemplateOrgService.findTemplateInfoByOrgIds(orgIds);
        templateInfoList = templateInfoList.stream().distinct().collect(Collectors.toList());
        System.out.println(JSONObject.toJSONString(templateInfoList));
        Map<String, Object> result = new HashMap<>();

        // 变量印品
        List<Map<String, Object>> variable = new ArrayList<>();
        List<TemplatePrintInfoResult> variableList = templateInfoList.stream()
                .filter(e -> TemplateTypeEnum.VARIABLE.equals(e.getTemplateType()))
                .collect(Collectors.toList());
        for (ClassifyEnum value : ClassifyEnum.values()) {
            Map<String, Object> variableMap = new HashMap<>();
            List<TemplatePrintInfoResult> sourceList = variableList.stream().filter(e -> value.equals(e.getTemplateClassify())).collect(Collectors.toList());
            List<Map<String, Object>> templateList = new ArrayList<>();
            for (TemplatePrintInfoResult templatePrintInfoResult : sourceList) {
                Map<String, Object> printMap = new HashMap<>();
                printMap.put("id", String.valueOf(templatePrintInfoResult.getTemplateId()));
                printMap.put("name", templatePrintInfoResult.getTemplateName());
                printMap.put("attachmentId", templatePrintInfoResult.getAttachmentId().toString());
                templateList.add(printMap);
            }
            if (templateList.size() > 0) {
                variableMap.put("type", value);
                variableMap.put("template", templateList);
                variable.add(variableMap);
            }
        }

        // 普通印品
        List<Map<String, Object>> ordinary = new ArrayList<>();
        List<TemplatePrintInfoResult> ordinaryList = templateInfoList.stream()
                .filter(e -> TemplateTypeEnum.ORDINARY.equals(e.getTemplateType()))
                .collect(Collectors.toList());
        Map<String, Object> ordinaryMap = new HashMap<>();
        for (ClassifyEnum value : ClassifyEnum.values()) {
            List<TemplatePrintInfoResult> sourceList = ordinaryList.stream().filter(e -> value.equals(e.getTemplateClassify())).collect(Collectors.toList());
            List<Map<String, Object>> templateList = new ArrayList<>();
            for (TemplatePrintInfoResult templatePrintInfoResult : sourceList) {
                Map<String, Object> printMap = new HashMap<>();
                printMap.put("id", String.valueOf(templatePrintInfoResult.getTemplateId()));
                printMap.put("name", templatePrintInfoResult.getTemplateName());
                printMap.put("attachmentId", templatePrintInfoResult.getAttachmentId().toString());
                templateList.add(printMap);
            }
            if (templateList.size() > 0) {
                ordinaryMap.put("type", value);
                ordinaryMap.put("template", templateList);
                ordinary.add(ordinaryMap);
            }
        }

        result.put("variable", variable);
        result.put("ordinary", ordinary);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean savePrintPlan(PrintPlanParams printPlanParams) {
        boolean result;
        System.out.println("JSON = " + printPlanParams.getOrdinaryContent());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long id = printPlanParams.getId();
        Long schoolId = printPlanParams.getSchoolId();
        if (basicSchoolService.list(new QueryWrapper<BasicSchool>().lambda().eq(BasicSchool::getId, schoolId).eq(BasicSchool::getEnable, true)).size() != 1) {
            throw ExceptionResultEnum.ERROR.exception("输入的学校id不满足条件");
        }

        // 校验是否印试卷和题卡
        List<String> printContentList = printPlanParams.getPrintContent();
        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(schoolId);
        if (Objects.isNull(basicExamRule)) {
            throw ExceptionResultEnum.ERROR.exception("未找到学校的通用规则");
        }
        /*if (basicExamRule.getIncludePaper() && !printContentList.contains("PAPER")) {
            throw ExceptionResultEnum.ERROR.exception("学校通用规则设置了强制包含试卷的必须选择试卷印品！");
        }*/
        if (printContentList.contains("PAPER") && !printContentList.contains("CARD")) {
            throw ExceptionResultEnum.ERROR.exception("如果试卷题卡印品选择了试卷，题卡必须被选择");
        }

        ExamPrintPlan examPrintPlan = new ExamPrintPlan();
        BeanUtils.copyProperties(printPlanParams, examPrintPlan);
        String printContent = StringUtils.join(printContentList, ",");
        examPrintPlan.setPrintContent(printContent);
        examPrintPlan.setCreateId(sysUser.getId());
        examPrintPlan.setUpdateId(sysUser.getId());
        if (id == null || id == 0) {
            // 没有印刷计划id -> 新增印刷计划
            examPrintPlan.setId(SystemConstant.getDbUuid());
            examPrintPlan.setStatus(PrintPlanStatusEnum.NEW);
            result = this.save(examPrintPlan);
        } else {
            // 包含印刷计划id -> 编辑印刷计划
            ExamPrintPlan tmp = this.getById(id);
            if (tmp == null) {
                throw ExceptionResultEnum.ERROR.exception("印刷计划id不存在");
            }
            PrintPlanStatusEnum status = tmp.getStatus();
            if (!PrintPlanStatusEnum.NEW.equals(status) && !PrintPlanStatusEnum.READY.equals(status)) {
                throw ExceptionResultEnum.ERROR.exception("已经开始打印的印刷计划不能编辑");
            }
            // 学校id不能更改
            examPrintPlan.setSchoolId(tmp.getSchoolId());
            result = this.updateById(examPrintPlan);
        }
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removePrintPlan(DeleteParams deleteParams) {
        Long id = deleteParams.getId();
        ExamPrintPlan examPrintPlan = this.getById(id);
        if (examPrintPlan == null) {
            throw ExceptionResultEnum.ERROR.exception("要删除的项目计划不存在");
        }
        if (!PrintPlanStatusEnum.NEW.equals(examPrintPlan.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("只有'新建'状态的项目计划可以被删除");
        }
        if (!tbTaskService.countByPrintPlanIdAndEntityId(examPrintPlan.getSchoolId(), id, null)) {
            throw ExceptionResultEnum.ERROR.exception("要删除的印刷计划正在生成pdf中，无法删除");
        }
        examDetailService.deleteExaminationData(id);
        return this.removeById(id);
    }

    @Override
    public List<Map<String, String>> tempalteView(Long printPlanId) {
        ExamPrintPlan examPrintPlan = this.getById(printPlanId);
        String variableContent = examPrintPlan.getVariableContent();
        List<Map<String, String>> finalList = new ArrayList<>();
        List<Map> variableList = JSONObject.parseArray(variableContent, Map.class);
        for (Map variable : variableList) {
            finalList.add(getAttachment(variable));
        }

        String ordinaryContent = examPrintPlan.getOrdinaryContent();
        List<Map> ordinaryList = JSONObject.parseArray(ordinaryContent, Map.class);
        for (Map ordinary : ordinaryList) {
            finalList.add(getAttachment(ordinary));
        }
        return finalList;
    }

    @Override
    public IPage<ClientPrintTaskDto> listClientPrintTask(Page<ClientPrintTaskDto> page, Long schoolId, Long machineCode, String printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Boolean isDownload, Boolean validate, Set<Long> orgIds) {
        // 以下状态考场状态不可查询
        String[] examDetailStatus = new String[]{ExamDetailStatusEnum.NEW.name(), PrintPlanStatusEnum.READY.name()};
        return this.baseMapper.listClientPrintTask(page, schoolId, machineCode, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime, isDownload, validate, orgIds, examDetailStatus);
    }


    @Override
    public List<ClientPrintTaskDto> listClientPrintTask(Long schoolId, Long machineCode, String printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Boolean isDownload, Boolean validate, Set<Long> orgIds) {
        // 以下状态考场状态不可查询
        String[] examDetailStatus = new String[]{ExamDetailStatusEnum.NEW.name(), PrintPlanStatusEnum.READY.name()};
        return this.baseMapper.listClientPrintTask(schoolId, machineCode, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime, isDownload, validate, orgIds, examDetailStatus);
    }

    @Override
    public ClientPrintTaskTotalDto clientTaskTotalData(String printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Boolean isDownload, Boolean validate, Long machineCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        // 以下状态考场状态不可查询
        String[] examDetailStatus = new String[]{ExamDetailStatusEnum.NEW.name(), PrintPlanStatusEnum.READY.name()};
        ClientPrintTaskTotalDto clientPrintTaskTotalDto = this.baseMapper.clientTaskTotalData(schoolId, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime, isDownload, validate, machineCode, orgIds, examDetailStatus);

        if (clientPrintTaskTotalDto != null) {
            // 累计a3,a4
            List<ClientPrintData> list = clientPrintDataService.listBySchoolIdAndMachineCode(schoolId, machineCode);
            if (!list.isEmpty()) {
                int pagesA3 = list.stream().mapToInt(m -> m.getPagesA3()).sum();
                clientPrintTaskTotalDto.setHostPagesA3(pagesA3 / 2);
                int pagesA4 = list.stream().mapToInt(m -> m.getPagesA4()).sum();
                clientPrintTaskTotalDto.setHostPagesA4(pagesA4 / 2);
            }
        }

        return clientPrintTaskTotalDto;
    }

    @Override
    public IPage<ClientPrintStatisticsDto> listClientPrintStatistics(Page<ClientPrintStatisticsDto> page, Long schoolId, String printPlanId, String examPlace, Long examStartTime, Long examEndTime, String courseCode, String paperNumber, Set<Long> orgIds) {
        // 以下状态考场状态不可查询
        String[] examDetailStatus = new String[]{ExamDetailStatusEnum.NEW.name(), PrintPlanStatusEnum.READY.name()};
        return this.baseMapper.listClientPrintStatistics(page, schoolId, printPlanId, examPlace, examStartTime, examEndTime, courseCode, paperNumber, orgIds, examDetailStatus);
    }

    @Override
    public ClientPrintStatisticsTotalDto clientStatisticsTotalData(String printPlanId, String examPlace, Long examStartTime, Long examEndTime, String courseCode, String paperNumber) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        // 以下状态考场状态不可查询
        String[] examDetailStatus = new String[]{ExamDetailStatusEnum.NEW.name(), PrintPlanStatusEnum.READY.name()};
        ClientPrintStatisticsTotalDto clientPrintStatisticsTotalDto = this.baseMapper.clientStatisticsTotalData(schoolId, printPlanId, examPlace, examStartTime, examEndTime, courseCode, paperNumber, orgIds, examDetailStatus);

        if (clientPrintStatisticsTotalDto != null) {
            String paperNumberStrs = clientPrintStatisticsTotalDto.getPaperNumberAndType();
            if(StringUtils.isNotBlank(paperNumberStrs)){
                String[] paperNumbers = paperNumberStrs.split(",");
                Set<String> paperNumberSet = new HashSet<>(Arrays.asList(paperNumbers));
                clientPrintStatisticsTotalDto.setPaperCount(paperNumberSet.size());
            }
        }

        return clientPrintStatisticsTotalDto;
    }


    /**
     * 查找子机构
     *
     * @param ids        上级机构id集合
     * @param schoolId   学校id
     * @param resultList 已查询所有子机构id集合
     * @return 所有子机构id集合
     */
    private List<Long> dp(List<Long> ids, Long schoolId, List<Long> resultList) {
        resultList.addAll(ids);
        for (Long id : ids) {
            List<SysOrg> childList = sysOrgService.list(new QueryWrapper<SysOrg>().lambda().eq(SysOrg::getParentId, id).eq(SysOrg::getSchoolId, schoolId));
            if (childList.size() > 0) {
                List<Long> childIds = childList.stream().map(SysOrg::getId).collect(Collectors.toList());
                this.dp(childIds, schoolId, resultList);
            }
        }
        return resultList;
    }

    public Map<String, String> getAttachment(Map map) {
        Map<String, String> fMap = new HashMap<>();
        fMap.put("type", map.get("type").toString());
        String attachmentId = map.get("attachmentId").toString();
        BasicAttachment attachment = basicAttachmentService.getById(attachmentId);
        if (attachment.getType().equals(".ftl") || attachment.getType().equals(".html")) {
            String content = commonService.readFileContent(attachment.getPath());
            fMap.put("content", content);
            fMap.put("utl", null);
        } else if (attachment.getType().equals(".pdf")) {
            fMap.put("content", null);
            Map<String, String> stringMap = commonService.filePreviewByAttachmentId(Long.valueOf(attachmentId), false);
            fMap.putAll(stringMap);
        } else {
            throw ExceptionResultEnum.ERROR.exception("不支持的类型");
        }
        return fMap;
    }
}
