package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailDto;
import com.qmth.distributed.print.business.bean.params.RelatePaperParam;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.TaskTypeEnum;
import com.qmth.distributed.print.business.mapper.ExamTaskDetailMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncCreatePdfTempleteService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题任务明细表（入库申请） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskDetailServiceImpl extends ServiceImpl<ExamTaskDetailMapper, ExamTaskDetail> implements ExamTaskDetailService {

    @Autowired
    private ExamTaskService examTaskService;

    @Autowired
    private ExamDetailService examDetailService;

    @Autowired
    private ExamDetailCourseService examDetailCourseService;

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private CommonService commonService;

    @Resource
    private AsyncCreatePdfTempleteService asyncCreatePdfTempleteService;

    @Autowired
    private ExamPrintPlanService examPrintPlanService;

    @Resource
    TBTaskService tbTaskService;

    @Autowired
    private ExamTaskPaperLogService examTaskPaperLogService;

    @Transactional
    @Override
    public boolean enable(ExamTaskDetail examTaskDetail) {
        // 禁用，解决试卷关联关系，并且解除生成pdf绑定
        if (!examTaskDetail.getEnable()) {
            // 正在生成pdf不能禁用
            ExamTask examTask = examTaskService.getById(examTaskDetail.getId());
            List<ExamDetail> examDetails = examDetailService.listByCourseCodeAndPaperNumber(examTask.getSchoolId(), examTask.getCourseCode(), examTask.getPaperNumber());
            if (examDetails != null && examDetails.size() > 0) {
                Map<Long, List<Long>> map = examDetails.stream().collect(Collectors.groupingBy(ExamDetail::getPrintPlanId, Collectors.mapping(ExamDetail::getId, Collectors.toList())));
                for (Map.Entry<Long, List<Long>> listEntry : map.entrySet()) {
                    boolean b = tbTaskService.countByPrintPlanIdAndEntityId(examTask.getSchoolId(), listEntry.getKey(), listEntry.getValue());
                    if (!b) {
                        throw ExceptionResultEnum.ERROR.exception("有关联该任务的考场正在生成打印pdf，不能禁用卷库");
                    }

                    // 解除所有考场该任务绑定关系
                    // 更新exam_detail
                    List<Long> examDetailIds = listEntry.getValue();
                    UpdateWrapper<ExamDetail> examDetailUpdateWrapper = new UpdateWrapper<>();
                    examDetailUpdateWrapper.lambda()
                            .set(ExamDetail::getStatus, ExamDetailStatusEnum.NEW)
                            .set(ExamDetail::getAttachmentId, null)
                            .set(ExamDetail::getAttachmentPath, null)
                            .set(ExamDetail::getPagesA3, null)
                            .set(ExamDetail::getPagesA4, null)
                            .in(ExamDetail::getId, examDetailIds);
                    examDetailService.update(examDetailUpdateWrapper);
                    // 更新绑定关系
                    List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByPrintPlanIdAndCourseCodeAndPaperNumber(examTask.getSchoolId(), listEntry.getKey(), examTask.getCourseCode(), examTask.getPaperNumber());
                    List<Long> examDetailCourseIds = examDetailCourses.stream().map(m -> m.getId()).collect(Collectors.toList());
                    UpdateWrapper<ExamDetailCourse> examDetailCourseUpdateWrapper = new UpdateWrapper<>();
                    examDetailCourseUpdateWrapper.lambda()
                            .set(ExamDetailCourse::getPaperType, null)
                            .set(ExamDetailCourse::getPaperPagesA3, null)
                            .set(ExamDetailCourse::getCardPagesA3, null)
                            .in(ExamDetailCourse::getId, examDetailCourseIds);
                    examDetailCourseService.update(examDetailCourseUpdateWrapper);

                    // 全部任务都取消，更新计划状态
                    //所有考场都撤回，印刷任务状态改为就绪
                    QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
                    examDetailQueryWrapper.lambda().eq(ExamDetail::getPrintPlanId, listEntry.getKey()).notIn(ExamDetail::getStatus, ExamDetailStatusEnum.NEW, ExamDetailStatusEnum.READY);
                    List<ExamDetail> examDetailList = examDetailService.list(examDetailQueryWrapper);
                    if (examDetailList.isEmpty()) {
                        UpdateWrapper<ExamPrintPlan> printPlanUpdateWrapper = new UpdateWrapper<>();
                        printPlanUpdateWrapper.lambda().set(ExamPrintPlan::getStatus, PrintPlanStatusEnum.READY).eq(ExamPrintPlan::getId, listEntry.getKey());
                        examPrintPlanService.update(printPlanUpdateWrapper);
                    }
                }
            }
        }
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getEnable, examTaskDetail.getEnable()).eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getId());
        this.update(updateWrapper);

        return true;
    }

    @Transactional
    @Override
    public boolean updatePaper(RelatePaperParam paperParam) throws IOException {
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(paperParam.getPrintPlanId());
        if (PrintPlanStatusEnum.END.equals(examPrintPlan.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划已结束，不能修改");
        }
        ExamTask examTask = examTaskService.getById(paperParam.getExamTaskId());

        // 提交印刷的考场，不允许修改关联试卷
        List<ExamDetail> examDetails = examDetailService.listByPrintPlanIdAndCourseCodeAndPaperNumber(examTask.getSchoolId(), paperParam.getPrintPlanId(), examTask.getCourseCode(), examTask.getPaperNumber());
        examDetails = examDetails.stream().filter(m -> !ExamDetailStatusEnum.NEW.equals(m.getStatus()) && !ExamDetailStatusEnum.READY.equals(m.getStatus())).collect(Collectors.toList());
        if (!examDetails.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("有考场已开始打印，不能修改");
        }

        // 更新关联卷型
        ExamTask newExamTask = examTaskService.getByCourseCodeAndPaperNumber(examTask.getSchoolId(), examTask.getCourseCode(), paperParam.getPaperNumber());
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getRelatePaperType, paperParam.getRelatePaperType()).eq(ExamTaskDetail::getExamTaskId, newExamTask.getId());
        this.update(updateWrapper);

        // 更新试卷编号
        String examDetailCourseIdStr = paperParam.getExamDetailCourseIds();
        List<Long> examDetailCourseIds = Arrays.asList(examDetailCourseIdStr.split(",")).stream().map(m -> Long.valueOf(m)).collect(Collectors.toList());
//        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByPrintPlanIdAndCourseCodeAndPaperNumber(examTask.getSchoolId(), paperParam.getPrintPlanId(), examTask.getCourseCode(), examTask.getPaperNumber());
        examDetailCourseService.updatePaperNumber(examDetailCourseIds, paperParam.getPaperNumber(), paperParam.getRelatePaperType());

        // 更新考生关联类型
        examStudentService.updatePaperType(examDetailCourseIds, paperParam.getRelatePaperType());

        // 重新生成pdf
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByIds(examDetailCourseIds);
        Map<Long, List<Long>> examDetailIdMap = examDetailCourses.stream().collect(Collectors.groupingBy(ExamDetailCourse::getExamDetailId, Collectors.mapping(ExamDetailCourse::getId, Collectors.toList())));
        for (Map.Entry<Long, List<Long>> listEntry : examDetailIdMap.entrySet()) {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.CREATE_PDF, paperParam.getPrintPlanId(), sysUser, listEntry.getKey());
            map.computeIfAbsent("examDetailCourseIds", v -> listEntry.getValue());
            map.computeIfAbsent("paperType", v -> paperParam.getRelatePaperType());
            asyncCreatePdfTempleteService.createPdf(map, null);
        }
        return true;
    }

    @Override
    public IPage<ExamTaskDetailDto> list(String relateType, Long printPlanId, String courseCode, String paperNumber, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDetailDto> examDetailDtoIPage = this.baseMapper.listPage(page, schoolId, relateType, printPlanId, courseCode, paperNumber, orgIds);
        return examDetailDtoIPage;
    }

    @Override
    public List<ExamTaskDetail> listByCardId(Long cardId) {
        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getCardId, cardId);
        return this.list(queryWrapper);
    }

    @Override
    public void bindCardId(String examTaskId, Long cardId) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getCardId, cardId).eq(ExamTaskDetail::getExamTaskId, examTaskId);
        this.update(updateWrapper);
    }

    @Override
    public void resetCardId(Long id) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getCardId, null).eq(ExamTaskDetail::getExamTaskId, id);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamTaskDetail> listByTemplateId(Long templateId) {
        return this.baseMapper.listByTemplateId(templateId);
    }

    @Override
    public Map<String, String> getUrl(Long schoolId, Long examTaskId) {
        QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
        examTaskDetailQueryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskId);
        ExamTaskDetail examTaskDetail = this.getOne(examTaskDetailQueryWrapper);
        if (examTaskDetail != null) {
            String relatePaperType = examTaskDetail.getRelatePaperType();
            if (StringUtils.isNotBlank(relatePaperType)) {
                String paperAttachmentIds = examTaskDetail.getPaperAttachmentIds();
                List<Map> list = JSONObject.parseArray(paperAttachmentIds, Map.class);
                String attachmentId = null;
                for (Map map : list) {
                    String name = map.get("name").toString();
                    if (relatePaperType.equals(name)) {
                        attachmentId = map.get("attachmentId").toString();
                    }
                }
                if (StringUtils.isNotBlank(attachmentId)) {
                    return commonService.filePreviewByAttachmentId(Long.valueOf(attachmentId), false);
                }
            }
        }
        return null;
    }

    @Override
    public boolean paperUpdate(ExamTaskDetail examTaskDetail) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // todo 前置校验条件，如打印中不能修改
        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getExamTaskId());
        ExamTaskDetail examTaskDetailTemp = this.getOne(queryWrapper);
        if (examTaskDetailTemp == null) {
            throw ExceptionResultEnum.ERROR.exception("命题任务数据异常");
        }
        // 已曝光试卷不能修改
        String exposedPaperType = examTaskDetailTemp.getExposedPaperType();
        if (StringUtils.isNotBlank(exposedPaperType)) {
            List<Map> oldPaperAttachmentIdsList = JSONObject.parseArray(examTaskDetailTemp.getPaperAttachmentIds(), Map.class);
            Map<String, String> oldPaperAttachmentIdMap = oldPaperAttachmentIdsList.stream().collect(Collectors.toMap(m -> m.get("name").toString(), m -> m.get("attachmentId").toString()));
            String[] exposedPaperTypes = exposedPaperType.split(",");
            String newPaperAttachmentIds = examTaskDetail.getPaperAttachmentIds();
            if (StringUtils.isBlank(newPaperAttachmentIds)) {
                throw ExceptionResultEnum.ERROR.exception("已曝光试卷不能删除");
            } else {
                List<Map> newPaperAttachmentIdsList = JSONObject.parseArray(newPaperAttachmentIds, Map.class);
                Map<String, String> newPaperAttachmentIdMap = newPaperAttachmentIdsList.stream().collect(Collectors.toMap(m -> m.get("name").toString(), m -> m.get("attachmentId").toString()));
                for (String paperType : exposedPaperTypes) {
                    if (!newPaperAttachmentIdMap.containsKey(paperType)) {
                        throw ExceptionResultEnum.ERROR.exception("已曝光试卷不能删除");
                    } else {
                        if (!oldPaperAttachmentIdMap.containsKey(paperType)) {
                            throw ExceptionResultEnum.ERROR.exception("原试卷卷型有误");
                        }
                        String newAttachmentId = newPaperAttachmentIdMap.get(paperType);
                        String oldAttachmentId = oldPaperAttachmentIdMap.get(paperType);
                        if (!newAttachmentId.equals(oldAttachmentId)) {
                            throw ExceptionResultEnum.ERROR.exception("已曝光试卷不能修改");
                        }
                    }
                }

            }

        }

        // 不需要审核，直接更新
        ExamTask examTask = examTaskService.getById(examTaskDetail.getExamTaskId());
        if (examTask.getReview()) {
            // 加入临时审核表
            ExamTaskPaperLog examTaskPaperLog = examTaskPaperLogService.getByExamTaskIdAndReview(examTaskDetail.getExamTaskId(), false);
            if (examTaskPaperLog == null) {
                examTaskPaperLog = new ExamTaskPaperLog();
                examTaskPaperLog.setId(SystemConstant.getDbUuid());
                examTaskPaperLog.setExamTaskId(examTaskDetail.getExamTaskId());
                examTaskPaperLog.setReview(false);
            }
            examTaskPaperLog.setPaperType(examTaskDetail.getPaperType());
            examTaskPaperLog.setPaperAttachmentIds(examTaskDetail.getPaperAttachmentIds());
            examTaskPaperLog.setCardId(examTaskDetail.getCardId());
            examTaskPaperLog.setCreateId(sysUser.getId());
            examTaskPaperLog.setCreateTime(System.currentTimeMillis());
            examTaskPaperLogService.saveOrUpdate(examTaskPaperLog);
        } else {
            UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamTaskDetail::getPaperAttachmentIds, examTaskDetail.getPaperAttachmentIds()).eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getExamTaskId());
            this.update(updateWrapper);
        }
        return true;
    }

    @Override
    public ExamTaskDetail getByExamTaskId(Long examTaskId) {
        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskId);
        return this.getOne(queryWrapper);
    }

}
