package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.params.ArraysParams;
import com.qmth.distributed.print.business.bean.result.TaskListResult;
import com.qmth.distributed.print.business.config.DictionaryConfig;
import com.qmth.distributed.print.business.entity.BasicAttachment;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.entity.TBTask;
import com.qmth.distributed.print.business.enums.TaskResultEnum;
import com.qmth.distributed.print.business.enums.TaskStatusEnum;
import com.qmth.distributed.print.business.enums.TaskTypeEnum;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import com.qmth.distributed.print.business.mapper.TBTaskMapper;
import com.qmth.distributed.print.business.service.BasicAttachmentService;
import com.qmth.distributed.print.business.service.SysUserService;
import com.qmth.distributed.print.business.service.TBTaskService;
import com.qmth.distributed.print.business.templete.execute.AsyncCreatePdfTempleteService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.JacksonUtil;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * <p>
 * 导入导出任务表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-03-19
 */
@Service
public class TBTaskServiceImpl extends ServiceImpl<TBTaskMapper, TBTask> implements TBTaskService {

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    TBTaskMapper tbTaskMapper;

    @Resource
    AsyncCreatePdfTempleteService asyncCreatePdfTempleteService;

    @Resource
    SysUserService sysUserService;

    @Resource
    DictionaryConfig dictionaryConfig;

    /**
     * 保存任务
     *
     * @param file
     * @param taskTypeEnum
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> saveTask(MultipartFile file, TaskTypeEnum taskTypeEnum) {
        BasicAttachment basicAttachment = null;
        Map<String, Object> map = null;
        try {
            basicAttachment = basicAttachmentService.saveAttachment(file, ServletUtil.getRequestMd5(), UploadFileEnum.FILE);
            if (Objects.isNull(basicAttachment)) {
                throw ExceptionResultEnum.ATTACHMENT_ERROR.exception();
            } else {
                map = new HashMap<>();
                TBTask tbTask = saveTaskCommon(basicAttachment, taskTypeEnum, map, null);
                this.save(tbTask);
            }
        } catch (Exception e) {
            log.error("请求出错", e);
            if (Objects.nonNull(basicAttachment)) {
                basicAttachmentService.deleteAttachment(basicAttachment);
            }
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Override
    @Transactional
    public Map<String, Object> saveTask(TaskTypeEnum taskTypeEnum) {
        Map<String, Object> map = null;
        try {
            map = new HashMap<>();
            TBTask tbTask = saveTaskCommon(null, taskTypeEnum, map, null);
            this.save(tbTask);
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Override
    @Transactional
    public Map<String, Object> saveTask(Long printPlanId, MultipartFile file, TaskTypeEnum taskTypeEnum) {
        BasicAttachment basicAttachment = null;
        Map<String, Object> map = null;
        try {
            basicAttachment = basicAttachmentService.saveAttachment(file, ServletUtil.getRequestMd5(), UploadFileEnum.FILE);
            if (Objects.isNull(basicAttachment)) {
                throw ExceptionResultEnum.ATTACHMENT_ERROR.exception();
            } else {
                map = new HashMap<>();
                TBTask tbTask = saveTaskCommon(basicAttachment, taskTypeEnum, map, null);
                tbTask.setPrintPlanId(printPlanId);
                this.save(tbTask);
            }
        } catch (Exception e) {
            log.error("请求出错", e);
            if (Objects.nonNull(basicAttachment)) {
                basicAttachmentService.deleteAttachment(basicAttachment);
            }
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    /**
     * 保存任务
     *
     * @param file
     * @param taskTypeEnum
     * @param objId
     * @return
     */
    @Override
    public Map<String, Object> saveTask(MultipartFile file, TaskTypeEnum taskTypeEnum, Object objId) {
        return null;
    }

    /**
     * 保存任务
     *
     * @param file
     * @param taskTypeEnum
     * @param projectId
     * @return
     */
    @Override
    public Map<String, Object> saveTask(MultipartFile file, TaskTypeEnum taskTypeEnum, Long projectId) {
        return null;
    }

    /**
     * 保存任务
     *
     * @param file
     * @param taskTypeEnum
     * @param objId
     * @param projectId
     * @return
     */
    @Override
    public Map<String, Object> saveTask(MultipartFile file, TaskTypeEnum taskTypeEnum, Object objId, Long projectId) {
        return null;
    }

    /**
     * 保存任务
     *
     * @param taskTypeEnum
     * @param printPlanId
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> saveTask(TaskTypeEnum taskTypeEnum, Long printPlanId) {
        Map<String, Object> map = new HashMap<>();
        try {
            TBTask tbTask = saveTaskCommon(null, taskTypeEnum, map, null);
            tbTask.setPrintPlanId(printPlanId);
            this.save(tbTask);
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    /**
     * 保存任务
     *
     * @param taskTypeEnum
     * @param printPlanId
     * @param sysUser
     * @param entityId
     * @return
     */
    @Override
    public Map<String, Object> saveTask(TaskTypeEnum taskTypeEnum, Long printPlanId, SysUser sysUser, Long entityId) {
        Map<String, Object> map = new HashMap<>();
        try {
            if (Objects.nonNull(entityId)) {
                map.computeIfAbsent("entityId", v -> entityId);
            }
            TBTask tbTask = saveTaskCommon(null, taskTypeEnum, map, sysUser);
            tbTask.setPrintPlanId(printPlanId);
            this.save(tbTask);
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    /**
     * 保存任务
     *
     * @param taskTypeEnum
     * @param arraysParams
     * @param sysUser
     * @return
     */
    @Override
    public Map<String, Object> saveTask(TaskTypeEnum taskTypeEnum, ArraysParams arraysParams, SysUser sysUser) {
        Map<String, Object> map = new HashMap<>();
        try {
            TBTask tbTask = saveTaskCommon(null, taskTypeEnum, map, sysUser);
            tbTask.setRemark(JacksonUtil.parseJson(arraysParams.getIds()));
            this.save(tbTask);
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    /**
     * 查询任务列表
     *
     * @param iPage
     * @param schoolId
     * @param printPlanId
     * @param status
     * @param type
     * @param result
     * @return
     */
    @Override
    public IPage<TaskListResult> query(IPage<Map> iPage, Long schoolId, Long printPlanId, TaskStatusEnum status, TaskTypeEnum type, TaskResultEnum result) {
        return tbTaskMapper.query(iPage, schoolId, printPlanId, Objects.nonNull(status) ? status.name() : null, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(result) ? result.name() : null);
    }

    /**
     * 获取任务是否完成
     *
     * @param schoolId
     * @param printPlanId
     * @param entityIds
     * @return
     */
    @Override
    public boolean countByPrintPlanIdAndEntityId(Long schoolId, Long printPlanId, List<Long> entityIds) {
        QueryWrapper<TBTask> tbTaskQueryWrapper = new QueryWrapper<>();
        tbTaskQueryWrapper.lambda().ne(TBTask::getStatus, TaskStatusEnum.FINISH).isNotNull(TBTask::getStatus)
                .eq(TBTask::getType, TaskTypeEnum.CREATE_PDF);
        if (Objects.nonNull(schoolId)) {
            tbTaskQueryWrapper.lambda().eq(TBTask::getSchoolId, schoolId);
        }
        if (Objects.nonNull(printPlanId)) {
            tbTaskQueryWrapper.lambda().eq(TBTask::getPrintPlanId, printPlanId);
        }
        if (Objects.nonNull(entityIds)) {
            tbTaskQueryWrapper.lambda().in(TBTask::getEntityId, entityIds);
        }
        return this.count(tbTaskQueryWrapper) == 0 ? true : false;
    }

    /**
     * 更新任务状态
     */
    @Override
    @Transactional
    public void updateStatus() {
        UpdateWrapper<TBTask> tbTaskQueryWrapper = new UpdateWrapper<>();
        tbTaskQueryWrapper.lambda().set(TBTask::getStatus, TaskStatusEnum.FINISH)
                .set(TBTask::getSummary, "系统提交:网络异常或程序异常中断!")
                .set(TBTask::getResult, TaskResultEnum.ERROR)
                .isNotNull(TBTask::getStatus).ne(TBTask::getStatus, TaskStatusEnum.FINISH);
        this.update(tbTaskQueryWrapper);
    }

    /**
     * 重新生成pdf失败任务
     *
     * @throws IOException
     */
    @Override
    @Transactional
    public void resetCreatePdfTask() throws IOException {
        QueryWrapper<TBTask> tbTaskQueryWrapper = new QueryWrapper<>();
        tbTaskQueryWrapper.lambda().eq(TBTask::getType, TaskTypeEnum.CREATE_PDF)
                .eq(TBTask::getStatus, TaskStatusEnum.FINISH).isNotNull(TBTask::getStatus)
                .eq(TBTask::getResult, TaskResultEnum.ERROR);
        List<TBTask> tbTasks = this.list(tbTaskQueryWrapper);
        if (Objects.nonNull(tbTasks) && tbTasks.size() > 0) {
            for (TBTask tbTask : tbTasks) {
                if (tbTask.getResetCount() < dictionaryConfig.sysDomain().getAutoCreatePdfResetMaxCount()) {
                    Map<String, Object> map = new HashMap<>();
                    tbTask.setVersion(new AtomicInteger(tbTask.getVersion()).incrementAndGet());
                    if (Objects.nonNull(tbTask.getRemark())) {
                        JSONArray jsonArray = JSONArray.parseArray(tbTask.getRemark());
                        ArraysParams arraysParams = new ArraysParams(jsonArray.toArray(new Long[jsonArray.size()]));
                        map.computeIfAbsent("examDetailCourseIds", v -> Arrays.asList(arraysParams.getIds()));
                    }
                    map.computeIfAbsent(SystemConstant.TASK, v -> tbTask);
                    map.computeIfAbsent(SystemConstant.USER, v -> sysUserService.getById(tbTask.getCreateId()));
                    asyncCreatePdfTempleteService.createPdf(map, null);
                }
            }
        }
    }

    /**
     * 保存task
     *
     * @param basicAttachment
     * @param taskTypeEnum
     * @param map
     * @param sysUser
     * @return
     */
    public TBTask saveTaskCommon(BasicAttachment basicAttachment, TaskTypeEnum taskTypeEnum, Map map, SysUser sysUser) {
        SysUser requestUser = Objects.nonNull(sysUser) ? sysUser : (SysUser) ServletUtil.getRequestUser();

        TBTask tbTask = new TBTask(taskTypeEnum,
                TaskStatusEnum.INIT,
                Objects.nonNull(basicAttachment) ? basicAttachment.getName() : null,
                Objects.nonNull(basicAttachment) ? basicAttachment.getPath() : null,
                requestUser.getId(),
                requestUser.getSchoolId());
        if (Objects.nonNull(map.get("entityId"))) {
            tbTask.setEntityId((Long) map.get("entityId"));
        }
        map.computeIfAbsent(SystemConstant.TASK, v -> tbTask);
        map.computeIfAbsent(SystemConstant.USER, v -> requestUser);
        return tbTask;
    }
}
