package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.entity.BasicCourse;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.service.BasicCourseService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 课程表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "课程Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.basic}/course")
public class BasicCourseController {

    @Autowired
    private BasicCourseService basicCourseService;

    /**
     * 查询
     * @param code
     * @param name
     * @param pageNumber
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(@RequestParam(value = "code", required = false) String code,
                       @RequestParam(value = "name", required = false) String name,
                       @RequestParam Integer pageNumber,
                       @RequestParam Integer pageSize){
        IPage<BasicCourse> userDtos = basicCourseService.list(code, name, pageNumber, pageSize);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 模糊查询
     * @param param
     * @return
     */
    @ApiOperation(value = "模糊查询")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    public Result query(@RequestParam(value = "param", required = false) String param,
                        @RequestParam(value = "printPlanId", required = false) Long printPlanId){
        List<BasicCourse> userDtos = basicCourseService.list(param, printPlanId);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 新增/修改
     * @param course
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(@RequestBody BasicCourse course){
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        course.setSchoolId(schoolId);
        boolean isSuccess = basicCourseService.saveCourse(course,sysUser.getId());
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     * @param course
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(@RequestBody BasicCourse course){
        boolean isSuccess = basicCourseService.remove(course.getId());
        return ResultUtil.ok(isSuccess);
    }
}

